package inf119.gamelogic;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;
import inf119.LocOri;
import inf119.interfaces.CameraControllerIntf;
import inf119.interfaces.StateIntf;

public class MyxCameraController extends AbstractMyxSimpleBrick implements CameraControllerIntf {

	public static final IMyxName msg = MyxUtils.createName("CameraController");
	public static final IMyxName statemsg = MyxUtils.createName("state");
	
	LocOri cam;
	double dist,  // length of the radius of the camera sphere
		angleI,	  // camera angle with regard to XZ plane
		angleR;   // camera rotation about the Y axis
	double[] pos; // center of the camera sphere
	boolean firstPersonMode = false, followCam = false;
	
	StateIntf gsc;
	
	public void begin(){
		gsc = (StateIntf)MyxUtils.getFirstRequiredServiceObject(this, statemsg);
		gsc.begin();
		LocOri c = gsc.getCameraInfo();
		double[] p = c.getPos(),
			rot = c.getRot();

		dist = 5;
		angleI = rot[0];
		angleR = rot[1];
		cam = new LocOri();		
		pos = new double[3];
		pos[0] = p[0];
		pos[1] = p[1];
		pos[2] = p[2];
		calcLocOri();
	}

	private double d2r(double degrees)
	{
		return (degrees / 180) * Math.PI;
	}
	
	private double r2d(double radians)
	{
		return (radians / Math.PI) * 180;
	}
	
	private void calcLocOri()
	{		
		double a1, a2, r1, r2;
		double[] newPos = new double[3], newOri = new double[3];
		
		r1 = dist;
		if (firstPersonMode)
			r1 = 0;
		
		
		//convert degree measurements to radians
		a1 = d2r(angleI);
		a2 = d2r(angleR);

		if (followCam == true){ //Use the follow cam
			/*
			 * NOTE:  This doesn't actually work properly!  
			 * It works for one axis, sorta for two, and not at all for three.
			 */
			
			
			//Ignore the user supplied values and instead lock onto the lander
			double[] shipRotation = gsc.getRotation(); //Ship rotation stores radian values
			
			newOri[0] = r2d(shipRotation[0]);
			newOri[1] = r2d(shipRotation[1]);
			newOri[2] = r2d(shipRotation[2]);
			System.out.println("newOri = {" + newOri[0] +","+ newOri[1] +","+ newOri[2]+"}");
			
			//calculate trig functions of rotation angles
			double sinRx = Math.sin(shipRotation[0]), cosRx = Math.cos(shipRotation[0]);
			double sinRy = Math.sin(shipRotation[1]), cosRy = Math.cos(shipRotation[1]);
			double sinRz = Math.sin(shipRotation[2]), cosRz = Math.cos(shipRotation[2]);
			
			//place camera at dist along the local z axis of the lander
			//do this through three separate rotations, precalculated here
			newPos[0] = pos[0] - dist * (cosRx * sinRy * cosRz - sinRx * sinRz);
			newPos[1] = pos[1] + dist * (cosRx * sinRy * sinRz + sinRx * cosRz);
			newPos[2] = pos[2] + dist * (cosRx * cosRy);			
			
			System.out.println("newPos = {" + newPos[0] +","+ newPos[1] +","+ newPos[2]+"}");
/*			
			newPos[0] = pos[0] + dist * Math.cos(shipRotation[0])*Math.sin(shipRotation[1]);
			newPos[1] = pos[1] - dist * Math.sin(shipRotation[0]);
			newPos[2] = pos[2] + dist*Math.cos(shipRotation[0])*Math.cos(shipRotation[1]);
			
			newOri[0] = 360 - r2d(shipRotation[0]);
			newOri[1] = 360 - r2d(shipRotation[1]);
			newOri[2] = 0 ;//360 - r2d(shipRotation[2]);
			
			System.out.println("newPos: " + newPos[0] + "," + newPos[1] + "," + newPos[2]);
			System.out.println("newOri: " + newOri[0] + "," + newOri[1] + "," + newOri[2]);
			System.out.println("shipOri: " + shipRotation[0] + "," + shipRotation[1] + "," + shipRotation[2]);
*/
		}
		else { //Use the camera sphere
			//calculate the height of the camera in the camera sphere		
			newPos[1] = r1 * Math.sin(a1) + pos[1];
			//calculate the radius of the slice of the camera sphere at that height
			r2 = r1 * Math.sin(Math.PI/2 - a1);		
			//calculate the position on that slice
			newPos[0] = r2 * Math.cos(a2) + pos[0];
			newPos[2] = r2 * Math.sin(a2) + pos[2];
		
			
			
			//Set the orientation
			newOri[0] = r2d(a1); //angleI;
			newOri[1] = r2d(a2) - 90;
			newOri[2] = 0;
		}
		
		//	Set the location
		cam.setLoc(newPos);
		//  Set the orientation
		cam.setOri(newOri);
		
		setCameraInformation();
	}

	private void setCameraInformation()
	{
		gsc.setCameraInfo(cam);
	}

	public void setPosition(double[] p)
	{
		if (p.length < 3){
			// malformed input
			System.err.println("Error: attempting to set camera position with fewer than 3 coords");
			return; 
		}
		for (int i = 0; i < 3; i++)
			pos[i] = p[i];
		calcLocOri();
	}
	public void swivelCamInclination(double delta)
	//Spin the camera about the X axis delta degrees
	{
		angleI += delta;
		if (angleI > 90)
			angleI = 90;
		else if (angleI < -90)
			angleI = -90;
		calcLocOri();
	}	
	public void swivelCamRotation(double delta)
	//Spin the camera about the Y axis delta degrees 
	{
		angleR = (angleR + delta) % 360;
		calcLocOri();
	}
	public void changeDistance(double delta)
	{
		dist += delta;
		calcLocOri();
	}
	public void setFirstPerson(boolean first)
	{
		firstPersonMode = first;
	}
	public void setFollow(boolean follow)
	{
		followCam = follow;
	}
	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}
}
