package inf119.gamelogic;

import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.Math;

import javax.media.opengl.GLCanvas;

import com.sun.opengl.util.Animator;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

import inf119.BoundBox;
import inf119.Constants;
import inf119.TerrainMap;
import inf119.LanderModel;
import inf119.LandingZone;
import inf119.interfaces.*;


public class MyxGameStateInterpreter extends AbstractMyxSimpleBrick implements TickIntf{

	public static final IMyxName msg = MyxUtils.createName("tick");
	public static final IMyxName outmsg = MyxUtils.createName("graphics");
	public static final IMyxName statemsg = MyxUtils.createName("state");
	public static final IMyxName pmmsg = MyxUtils.createName("PhysicsModel");
	public static final IMyxName ccmsg = MyxUtils.createName("CameraController");
	public static final IMyxName mcmsg = MyxUtils.createName("MenuController");

	StateIntf GSC;
	GraphicsIntf GM;
	UIParser UIP;
	Frame gameFrame;
	GLCanvas canvas;
	CallPhysics CP;
	TerrainMap TM;
	
	PhysicsModelIntf pm;
	CameraControllerIntf cc;
	MenuControllerIntf mc;
	
	double depthTollerance;
	double slopeTollerance;
	double maxLandingVelocity;
	
	private static final int notInsideLandingSite = -1;
	
	// add GSC.saveHighScores() to all final states
	
	public MyxGameStateInterpreter()
	{
		depthTollerance =2;
		slopeTollerance = 3;
	}

	public void begin(){
		GSC = (StateIntf)MyxUtils.getFirstRequiredServiceObject(this, statemsg);
		GM = (GraphicsIntf)MyxUtils.getFirstRequiredServiceObject(this, outmsg);
		
		//GSC.begin();
		//maxLandingVelocity = setMaxLandingVelocity(40);
		
		pm = (PhysicsModelIntf)MyxUtils.getFirstRequiredServiceObject(this, pmmsg);
		cc = (CameraControllerIntf)MyxUtils.getFirstRequiredServiceObject(this, ccmsg);
		mc = (MenuControllerIntf)MyxUtils.getFirstRequiredServiceObject(this, mcmsg);
		
		CP = new CallPhysics(GSC, pm);
		UIP = new UIParser(GSC, CP, cc, mc);
	}
	
	private void initializeGraphics()
	{
		System.out.println("Initializing Graphics");
		try {
			System.loadLibrary("jogl");
		}
		catch (Exception e){
			System.out.println("Error: JOGL libraries not found");
			System.err.println(e.getMessage());
		}
		
		/* Initialize and display the Java Frame */
		gameFrame = new Frame("Lunar Lander");
		int m = GM.getWidth();
		int h = GM.getHeight();
        gameFrame.setSize(m , h );
        gameFrame.addKeyListener(UIP);
        gameFrame.addMouseListener(UIP);
        gameFrame.addMouseMotionListener(UIP);
        
        canvas = new GLCanvas();
        canvas.addMouseListener(UIP);
        canvas.addKeyListener(UIP);
        canvas.addMouseMotionListener(UIP);
        gameFrame.add(canvas);
        
		canvas.addGLEventListener(GM);
		final Animator animator = new Animator(canvas);
		gameFrame.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				animator.stop();
				System.exit(0);
			}
		});
		gameFrame.show();
		animator.start();
		
		System.out.println("Graphics Ready");
	}
	
	public void interpret(double interval) throws FileNotFoundException, IOException
	{
		
		int state = GSC.getGameState();
		
		if(state ==Constants.MENUSTATE)
		{	
//			System.out.println("MENU STATE");
		}
		else if(state ==Constants.LOADMAPSTATE)
		{
			System.out.println("LOAD MAP STATE");
			
			GM.setMap();
			System.out.println("Map Set");
			GM.setLanderModel(GSC.getLanderModel());
			System.out.println("LanderModel Set");
			initializeGraphics();
	        

			GM.setLander(GSC.getShipPosition(), GSC.getRotation()); 
			setHUD();
			GM.setCamera(GSC.getCameraInfo());
				
			state = Constants.LIVEGAMESTATE;
			GSC.updateGameState(state);
			System.out.println("Done with LOADINGMAPSTATE");
		}
		else if (state == Constants.PAUSESTATE)
		{
			//Game is paused if there are any menus active
			GM.setMenu(GSC.getMenu());
			
		}
		else if(state == Constants.LIVEGAMESTATE)
		{				
			maxLandingVelocity = setMaxLandingVelocity(30);
			TM = getTerrainMap();
			//do game things
			
			//if ship has touched down check
			// speed < X ,slope <.5 , no rocks  landing good
			//// draw ship
			//set var and go 2 state 5
			
			//else
			////  GM.toggleCrashed();
			// set var go 2 state 5
			
			//if ship has not touched down
			// draw ship
			
//			Updates Physics
			
			double Location[] = GSC.getShipPosition();
			double totalvelocity = totalvelocity();
			BoundBox bb = new BoundBox(GSC.getLanderModel());
			bb.updateBoxLoc(GSC.getShipPosition());
			double terrainHeight = TM.interpolateHeight(Location[0],Location[2]);
			boolean crash = bb.isColliding(terrainHeight);
			
			if(TM.safeZone(Location[0], Location[2], bb, depthTollerance))
			{
				
				if (Constants.NO_LANDING ==TM.isLanding(Location[0], Location[2], bb, depthTollerance, slopeTollerance))
				{
					Menu menu = GSC.getMenu();
					if (menu == null){
						CP.tick(interval);
						
						UIP.updateCamPos(); //TODO: check to see if this is necessary
						GM.setLander(GSC.getShipPosition(), GSC.getRotation()); 
						GM.setThrusting(GSC.getThrusting());
						setHUD();
						GM.setCamera(GSC.getCameraInfo());
					}
					GM.setMenu(GSC.getMenu());
					
					GSC.updateGameState(Constants.LIVEGAMESTATE);
				}
				else if(totalvelocity < maxLandingVelocity &&  Constants.SAFE_LANDING ==TM.isLanding(Location[0], Location[2], bb, depthTollerance, slopeTollerance))
				{
					//Should check velocity
					
					Menu menu = GSC.getMenu();
					if (menu == null){
						CP.tick(interval);
						
						UIP.updateCamPos(); //TODO: check to see if this is necessary
						GM.setLander(GSC.getShipPosition(), GSC.getRotation()); 
						GM.setThrusting(GSC.getThrusting());
						setHUD();
						GM.setCamera(GSC.getCameraInfo());
					}
					GM.setMenu(GSC.getMenu());
					System.out.println("Safe landing: s[1] = " + GSC.getShipPosition()[1]);
					GSC.updateGameState(Constants.END_SAFELANDING_GAMESTATE);
				}
				else
				{
					//unsafe landing method
					GSC.updateGameState(Constants.END_UNSAFELANDING_GAMESTATE);
				}
	
			}
			else
			{
				//Explosion method 
				//Might not need this
				GSC.updateGameState(Constants.END_DEATHLANDING_GAMESTATE);
				
			}
		}
		
		else if(state ==Constants.END_SAFELANDING_GAMESTATE)
		{
			Menu m;
			if(GSC.getMenuState()==Constants.SCENARIOMODE)
			{
				double score = calculateTotalScore();
				String levelName = GSC.getScenarioName();
				boolean highScore = GSC.setHighScore(levelName, score);
				
				if(highScore)
				{
					//tell use got high score and what it is
					MenuItem[] quitMenu = {new MenuItem('Q', "Quit Game", Menu.CENTER_ALIGN, Constants.QUITGAMESTATE)};
					m = new Menu("New High Score! (Score = " + score + ")", Menu.CENTER_ALIGN, quitMenu, GM.getFont());
					GSC.saveHighScores();
				}
				else
				{
					//tell user that they didnt get high score and what it is
					MenuItem[] quitMenu = {new MenuItem('Q', "Quit Game", Menu.CENTER_ALIGN, Constants.QUITGAMESTATE)};
					m = new Menu("You Landed Successfully! (Score = " + score + ")", Menu.CENTER_ALIGN, quitMenu, GM.getFont());
				}
			}
			else
			{
				//Practice Mode: Just print successful landing
				MenuItem[] quitMenu = {new MenuItem('Q', "Quit Game", Menu.CENTER_ALIGN, Constants.QUITGAMESTATE)};
				m = new Menu("You Landed Successfully!", Menu.CENTER_ALIGN, quitMenu, GM.getFont());				
			}
			
			System.out.println("***Exiting Game. Safe landing***");			
			GSC.setMenu(m);						
			GSC.updateGameState(Constants.PAUSESTATE);
//			System.exit(0);
		}
		else if(state ==Constants.END_UNSAFELANDING_GAMESTATE)
		{
			//display messaage saying landing site was unsuitable 
			System.out.println("***Exiting Game. unsafe landing ***");
			MenuItem[] quitMenu = {new MenuItem('Q', "Quit Game", Menu.CENTER_ALIGN, Constants.QUITGAMESTATE)};
			Menu m = new Menu("You Died (Crashed while velocity exceeded " + maxLandingVelocity + ")", Menu.CENTER_ALIGN, quitMenu, GM.getFont());
			GSC.setMenu(m);			
			GSC.updateGameState(Constants.PAUSESTATE);
//			System.exit(0);
		}
		else if(state ==Constants.END_DEATHLANDING_GAMESTATE)
		{
			//display messaage saying landing site was unsuitable 
			System.out.println("***Exiting Game. DEATH ***");
			MenuItem[] quitMenu = {new MenuItem('Q', "Quit Game", Menu.CENTER_ALIGN, Constants.QUITGAMESTATE)};
			Menu m = new Menu("You Died (Slope exceeded " + slopeTollerance + ")", Menu.CENTER_ALIGN, quitMenu, GM.getFont());
			GSC.setMenu(m);
			GSC.updateGameState(Constants.PAUSESTATE);
//			System.exit(0);
		}
		
		else if(state ==Constants.QUITGAMESTATE)
		{
			//display messaage when quitting game
			System.out.println("***Exiting Game. quit game ***");
			System.exit(0);
		}		
		
		else //extra for right now
		{
//			do game ending things
			
			// if crashed
//			display menu to restart game
			
			//if not
			//cal. score calculateTotalScore()
			//display score
			//display menu to restart game
			
			System.out.println("***Exiting Game. ELSE state***");
			System.exit(0);
		}
	}
	
	private void setHUD()
	{
		//Calculate the magnitude of the velocity from the x,y,z component velocities
		double[] vels = GSC.getVelocity(), pos = GSC.getShipPosition();		
		double velocity = Math.sqrt(vels[0]*vels[0] + vels[1]*vels[1] + vels[2]*vels[2]); 
		GM.setLanderVelocity(velocity);
	
		TerrainMap tm = GSC.getTerrainMap();
		double terrainHeight = tm.resolution*tm.interpolateHeight(pos[0]/tm.resolution, pos[2]/tm.resolution),
		aboveHeight =  pos[1] - terrainHeight - 1*GSC.getLanderModel().getHeight();
		
		GM.setLanderAltitude(aboveHeight);
		if (aboveHeight <= 0)
			GM.toggleCrashed();
		
		GM.setLanderFuel(GSC.getHUDFuelLevel());

	}
	
	
	private TerrainMap getTerrainMap()
	{
		return GSC.getTerrainMap();
	}
	
	
	private LanderModel getLanderModel()
	{
		return GSC.getLanderModel();
	}
	
	private double totalvelocity()
	{
		double speed[] = GSC.getVelocity();
		
		double x = speed[0];
		double y = speed[1];
		double z = speed[2];
		
		double totalVelocity = Math.sqrt((x*x)+(y*y)+(z*z));
		return totalVelocity;
	}
	
	private double setMaxLandingVelocity(double mlv)
	{
		if(GSC.getDifficulty()==Constants.HARDDIFFICULTY)
		{
			 
			return (mlv * Constants.HARDLANDINGPERECENTAGE);
		}
		else if(GSC.getDifficulty()==Constants.NORMALDIFFICULTY)
		{
			return (mlv * Constants.NORMALLANDINGPERECENTAGE);
		}
		else
		{
			return mlv;
		}
 
	}
	
	
	
	//Score methods
	public double calculateTotalScore()
	{
		//(fuel bonus + safety + site bonus)*level
		
		double score = 0;
		score = (calculateFuelBonus() + calculateSafetyBonus() + 
				calculateLandingSiteBonus()) * calculateDifficultyBonus();
		return Math.round(score);
	}
	
	public double calculateFuelBonus()
	{
		double fuelBonus = 0;
		double differenceInFuelPercent = GSC.getInitialFuelPercent() - GSC.getCurrentFuelLevelPercent();
		
		//is used cause % is taken out of 100
		fuelBonus = (100-differenceInFuelPercent) * Constants.FuelBonus;
		
		return fuelBonus;
		
	}
	

	
	public double calculateSafetyBonus()
	{
		
		double speed= totalvelocity();
		
		if(speed> (Constants.NORMALLANDINGPERECENTAGE * maxLandingVelocity))
		{
			return Constants.EASYLANDINGBONUS;
		}
		else if(speed<=(Constants.NORMALLANDINGPERECENTAGE * maxLandingVelocity) && speed>=(Constants.HARDLANDINGPERECENTAGE * maxLandingVelocity))
		{
			return Constants.NORMALLANDINGBONUS;
		}
		else
		{
			return Constants.HARDLANDINGBONUS;
		}

	}
	
	public double calculateLandingSiteBonus()
	{
		TerrainMap  tMap = getTerrainMap();
		
		double[] position = GSC.getShipPosition();
		int landingZoneNumber =  insideOfLandingZones(position[0], position[2]);
		
		if(landingZoneNumber==notInsideLandingSite)
		{
			return 0;
		}
		else
		{
			LandingZone[] zone = tMap.getLandingZones();

			return zone[landingZoneNumber].score;
		}
		
		//find what landing site you are in and return that bonus
		//need to get current (x,Z)
		//insideOfLandingZones(X, Z)	
	}
	
	public double calculateDifficultyBonus()
	{
		
		if(GSC.getDifficulty()==Constants.EASYDIFFICULTY)
		{
			return Constants.EASYDIFFICULTYBONUS;
		}
		else if(GSC.getDifficulty()==Constants.NORMALDIFFICULTY)
		{
			return Constants.NORMALDIFFICULTYBONUS;
		}
		else
		{
			return Constants.HARDDIFFICULTYBONUS;
		}
		
	}

	
	
	
	//Location methods
	private int insideOfLandingZones(double currentX, double currentZ) //number of landing zone ship is in.
	{
		TerrainMap  tMap = getTerrainMap();
		
		LandingZone[] landingZones = tMap.getLandingZones();
		int numberOfLandingZones = landingZones.length;
		
		
		int landingZoneNumber = notInsideLandingSite;
		
		for(int i=0; i<numberOfLandingZones; i++)
		{
			
			if(insideOfLandingZoneRange(landingZones[i].x,landingZones[i].z, 
					landingZones[i].radius, currentX, currentZ))
			{
				return i;
			}						
		}
		
		
		return landingZoneNumber;
		
	}
	
	private boolean insideOfLandingZoneRange(double lzX, double lzZ, double radius, double currentX, double currentZ)
	{
		double lowerBoundsX	= lzX - radius;
		double upperBoundsX = lzX + radius;
		
		double lowerBoundsY = lzZ - radius;
		double upperBoundsY = lzZ + radius;
		
		
		if(currentX>lowerBoundsX && currentX< upperBoundsX)
		{
			if(currentZ>lowerBoundsY && currentZ< upperBoundsY)
			{
				return true;
			}
		}
		
		return false;
	
	}
	
	public double distance(double[] location1, double[] location2)
	{
		
		double x1 = location1[0];
		double y1 = location1[1];
		double z1 = location1[2];
		
		double x2 = location2[0];
		double y2 = location2[1];
		double z2 = location2[2];
		
		double xDiff = x2- x1;
		double yDiff = y2- y1;
		double zDiff = z2- z1;
		
		
		double distance = ((xDiff*xDiff) + (yDiff*yDiff) + (zDiff*zDiff));
		
		if(distance<0)
		{
			distance = distance* -1;
		}
		distance= Math.sqrt(distance);
		
		return distance;
	}

	public void tick(double interval) {
		// TODO Auto-generated method stub
		//Interprets game state
		try {
			interpret(interval);			
		}		
		catch (Exception e)
		{
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}

}

