/**
 * UIParser.java
 * 
 * Parses user input and makes the appropriate in-game function calls
 * 
 * @author mmasukaw
 */

package inf119.gamelogic;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import inf119.Constants;
import inf119.KeyMap;
import inf119.MovementType;
import inf119.interfaces.*;

public class UIParser implements KeyListener, MouseListener, MouseMotionListener {
	final double ZOOM_MOUSE_SCALE = 10, ROTATE_MOUSE_SCALE = 0.2;
	CameraControllerIntf cc;
	MenuControllerIntf mc;
	KeyMap km;
	int lastX, lastY, button;
	StateIntf gsc;
	CallPhysics callPhys;

	public UIParser(StateIntf gsc, CallPhysics cp, CameraControllerIntf ca, MenuControllerIntf me)
	{
		this.gsc = gsc;
		cc = ca;
		km = new KeyMap();	
		mc = me;
		callPhys = cp;
	}
	
	private boolean isGameInProgress()
	{
		if (gsc.getGameState() == Constants.LIVEGAMESTATE){
			return true;
		}
		return false;
	}
	
	public void mousePressed(MouseEvent me)
	{			
		lastX = me.getX();
		lastY = me.getY();
		button = me.getButton();
	}
	
	public void mouseDragged(MouseEvent me)
	{
		if (gsc.getMenu() != null){
			//Disable mouse functionality during menu mode
			return;
		}
		//Note: upper left corner of screen is (0,0)
		int deltaX, deltaY, x = me.getX(), y = me.getY();
		deltaX = x - lastX;
		deltaY = y - lastY;
		
		switch(button){
		case (MouseEvent.BUTTON1): //left click		
			if (isGameInProgress() == true){
				//if the game is running, swivel the camera as desired
				cc.swivelCamRotation(deltaX*ROTATE_MOUSE_SCALE);
				cc.swivelCamInclination(deltaY*ROTATE_MOUSE_SCALE);
			}
			break;
		case (MouseEvent.BUTTON3): //right click
			if (isGameInProgress() == true){
				//if the game is running, swivel the camera as desired
				cc.changeDistance(deltaY*ZOOM_MOUSE_SCALE);
			}
			break;
			
		}
		lastX = x;
		lastY = y;
	}
	
	public void mouseMoved(MouseEvent me)	{
	}
	public void mouseReleased(MouseEvent me) {
	}

	private void displayMovements(boolean [] moves)
	{
		for (int i = 0; i < moves.length; i++ )
		{
			if (moves[i] == true)
				System.out.print(MovementType.iToS(i) + " ");
		}
		System.out.println();
	}
	
	public void keyPressed(KeyEvent ke) {
		int code = ke.getKeyCode();
		km.keyPressed(code);
		setActions(km.getPressedKeys());
		if (mc.checkMenuOption(code) == false){
			/* 
			 * There was not already a menu in operation,
			 * so see if we're supposed to open a new one 
			 */
			int menuCode = km.getMenuCode(code);		
			if (menuCode != 2) {//MyxMenuController.NO_CHANGE=2
				mc.setMenu(menuCode);
			}
		}
		/* checkMenuOption also executes the state change decreed by the menu */
	}

	public void keyReleased(KeyEvent ke) {
		km.keyReleased(ke.getKeyCode());
		setActions(km.getPressedKeys());
		if (km.getCameraMode() == KeyMap.FIRST_PERSON_CAM)
			cc.setFirstPerson(true);
		else if (km.getCameraMode() == KeyMap.THIRD_PERSON_CAM)
			cc.setFirstPerson(false);
				
		
		cc.setFollow(km.isFollow());
	}

	private void setActions(boolean[] moves)
	{
		//displayMovements(km.getPressedKeys());
		//Tell CallPhysics which actions the user intends to perform
		callPhys.setActions(moves);	
		//Update the camera's position
		updateCamPos();
		
	}
	
	public void updateCamPos()
	{
		//Set the center of the camera sphere to be the ship's location
		double[] position = gsc.getShipPosition();	
		cc.setPosition(position);
	}
	
	public void keyTyped(KeyEvent arg0) {
	}

	public void mouseClicked(MouseEvent me) {
	}

	public void mouseEntered(MouseEvent arg0) {
	}

	public void mouseExited(MouseEvent arg0) {
	}
}
