package inf119.graphics;

import javax.media.opengl.GL;

import inf119.interfaces.EffectsArtistIntf;

import javax.media.opengl.glu.GLU;
import javax.media.opengl.glu.GLUquadric;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

import inf119.interfaces.TextureIntf;

public class MyxEffectsArtist extends AbstractMyxSimpleBrick implements EffectsArtistIntf {
	
	public static final IMyxName msg = MyxUtils.createName("EffectsArtist");
	public static final IMyxName txtmsg = MyxUtils.createName("Texture");

	GL gl;
	GLU glu;
	TextureIntf tm;
	boolean hasCrashed = false;
	
	double THRUST_WIDTH = 0.75, THRUST_HEIGHT = 3.0, THRUST_OFFSET = 1.0;
	double[][][] THRUST_VERTICES =
		//Upsidedown tetrahedron flame
	{
			//Near face
			{{-THRUST_WIDTH, -THRUST_OFFSET, THRUST_WIDTH},
				{0, -THRUST_HEIGHT, 0},
				{THRUST_WIDTH, -THRUST_OFFSET, THRUST_WIDTH}},
			//Left face
			{{-THRUST_WIDTH, -THRUST_OFFSET, -THRUST_WIDTH},
				{0, -THRUST_HEIGHT, 0},
				{-THRUST_WIDTH, -THRUST_OFFSET, THRUST_WIDTH}},
			//Far face
			{{THRUST_WIDTH, -THRUST_OFFSET, -THRUST_WIDTH},
				{0, -THRUST_HEIGHT, 0},
				{-THRUST_WIDTH, -THRUST_OFFSET, -THRUST_WIDTH}},
			//Right face
			{{THRUST_WIDTH, -THRUST_OFFSET, THRUST_WIDTH},
				{0, -THRUST_HEIGHT, 0},
				{THRUST_WIDTH, -THRUST_OFFSET, -THRUST_WIDTH}},					
	};
	
	final double[][][] GUIDE_ARROW = 
	{
		{
			{0,0,0},
			{0,0.75,0},
			{0,0,-1},
			{0,-0.75,0}},
		{
			{0,0.25,0},
			{0,-0.25,0},
			{0,-0.25,1.5},
			{0,0.25,1.5}},	
		{
			{0,0,0},
			{0,-0.75,0},
			{0,0,-1},
			{0,0.75,0}},
		{
			{0,0.25,0},
			{0,0.25,1.5},
			{0,-0.25,1.5},
			{0,-0.25,0}},	
	};
	
	public MyxEffectsArtist()
	{
		glu = new GLU();
	}
	
	public void setGL(GL g)
	{
		gl = g;
	}
	
	
	public void setThrust(double width, double offset)
	{		
		double[][][] temp = {
				//Near face
				{{-width, offset, width},
					{0, offset-(width*2.5), 0},
					{width, offset, width}},
				//Left face
				{{-width, offset, -width},
					{0, offset-(width*2.5), 0},
					{-width, offset, width}},
				//Far face
				{{width, offset, -width},
					{0, offset-(width*2.5), 0},
					{-width, offset, -width}},
				//Right face
				{{width, offset, width},
					{0, offset-(width*2.5), 0},
					{width, offset, -width}},					
		};
		
		THRUST_VERTICES = temp;
	}
	
	//	this is used to make the thrust flicker
	private double scaleHeight()
	{ 
		return( 0.8 + Math.random() * 0.2);
	}
	
	private void drawThrust(double[] pos, double[] thrust, double[] rot, double res, double[] center)
	{
    	gl.glDisable( GL.GL_TEXTURE_2D );
		gl.glPushMatrix();
		
		//move to lander position
		double x = MyxGraphicsMaster.metersToUnits(pos[0]-center[0]),
				y = MyxGraphicsMaster.metersToUnits(pos[1]-center[1]),
				z = MyxGraphicsMaster.metersToUnits(pos[2]-center[2]);
		gl.glTranslated(x, y, z);
		
		//move to lander orientation
		gl.glRotated(-rot[0], 1.0, 0.0, 0.0);
		gl.glRotated(-rot[1], 0.0, 1.0, 0.0);
		gl.glRotated(-rot[2], 0.0, 0.0, 1.0);
		
		//shift to thruster offset
		gl.glTranslated(MyxGraphicsMaster.metersToUnits(thrust[0]),
				MyxGraphicsMaster.metersToUnits(thrust[1]),
				MyxGraphicsMaster.metersToUnits(thrust[2]));
		
		//scale based on lander model
		double newScale = MyxGraphicsMaster.metersToUnits(res);
		gl.glScaled(newScale, newScale, newScale);
				
		double height = scaleHeight();
		
		for (int i = 0; i < THRUST_VERTICES.length; i++){
			gl.glBegin(GL.GL_POLYGON);
				gl.glColor3d(1.0, 0.0, 0.0);
				gl.glVertex3d(THRUST_VERTICES[i][0][0], THRUST_VERTICES[i][0][1], THRUST_VERTICES[i][0][2]);
				gl.glColor3d(1.0, 1.0, 0.0);
				gl.glVertex3d(THRUST_VERTICES[i][1][0], height*THRUST_VERTICES[i][1][1], THRUST_VERTICES[i][1][2]);
				gl.glColor3d(1.0, 0.0, 0.0);
				gl.glVertex3d(THRUST_VERTICES[i][2][0], THRUST_VERTICES[i][2][1], THRUST_VERTICES[i][2][2]);
			gl.glEnd();
		}
		gl.glPopMatrix();
		gl.glEnable( GL.GL_TEXTURE_2D );
	}
	
	public void drawEffects(double[] shipPos, double[] rotation, int counter, boolean isThrusting, 
			boolean isCrashing, double[] thrustPos, double resolution, double[] center, boolean useGuide)
	{
		
		if (isThrusting == true)
		{			
			drawThrust(shipPos, thrustPos, rotation, resolution, center);
		}
		if( isCrashing )
		{
			if( counter > 358 )
				hasCrashed = true;
			if( !hasCrashed )
				drawExplosion(shipPos, (1.5/359.0)*counter);
		}
		if( useGuide )
			drawGuideArrow(shipPos, rotation, resolution);
		
	}
	
	private void drawExplosion(double[] pos, double scale)
	{
		gl.glDisable(GL.GL_DEPTH_TEST);
		gl.glEnable(GL.GL_BLEND);
        gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE);
        GLUquadric obj = glu.gluNewQuadric();
        gl.glPushMatrix();
        	double x = MyxGraphicsMaster.metersToUnits(pos[0]),
        			y = MyxGraphicsMaster.metersToUnits(pos[1]),
        			z = MyxGraphicsMaster.metersToUnits(pos[2]);        	
			gl.glTranslated(x, y, z);
			double alpha = (-0.8/1.5)*scale + 0.8; //makes the cloud disappear as it grows larger
			gl.glColor4d(0.5, 0.5, 0.5, alpha);
			glu.gluSphere(obj, scale, 16, 16); //draw dust cloud
        gl.glPopMatrix();
        glu.gluDeleteQuadric(obj);
        gl.glDisable(GL.GL_BLEND);
        gl.glEnable(GL.GL_DEPTH_TEST);
	}

	private void drawGuideArrow(double[] pos, double[] rot, double res)
	{
		gl.glPushMatrix();		
	
		//move to ship position
		double x = MyxGraphicsMaster.metersToUnits(pos[0]),
			y = MyxGraphicsMaster.metersToUnits(pos[1]),
			z = MyxGraphicsMaster.metersToUnits(pos[2]);
		gl.glTranslated(x, y, z);
		
		//move to ship orientation
		gl.glRotated(-rot[0], 1.0, 0.0, 0.0);
		gl.glRotated(-rot[1], 0.0, 1.0, 0.0);
		gl.glRotated(-rot[2], 0.0, 0.0, 1.0);
		
		//move to front of ship
		gl.glTranslated(0,0,-0.8);
		
		double scale = MyxGraphicsMaster.metersToUnits(res);
		gl.glScaled(scale, scale/2, scale);
		
		//draw arrow
		for (int face = 0; face < GUIDE_ARROW.length; face++){
			gl.glBegin(GL.GL_POLYGON);
			gl.glColor4d(0.0, 1.0, 0.0, 0.8);
			gl.glVertex3d(GUIDE_ARROW[face][0][0], GUIDE_ARROW[face][0][1], GUIDE_ARROW[face][0][2]);
			gl.glVertex3d(GUIDE_ARROW[face][1][0], GUIDE_ARROW[face][1][1], GUIDE_ARROW[face][1][2]);
			gl.glVertex3d(GUIDE_ARROW[face][2][0], GUIDE_ARROW[face][2][1], GUIDE_ARROW[face][2][2]);
			gl.glVertex3d(GUIDE_ARROW[face][3][0], GUIDE_ARROW[face][3][1], GUIDE_ARROW[face][3][2]);
			gl.glEnd();
		}
		
		gl.glPopMatrix();
	}
	
	public void begin(){
		tm = (TextureIntf)MyxUtils.getFirstRequiredServiceObject(this, txtmsg);
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}	
}
