package inf119.graphics;

import javax.media.opengl.GL;

import com.sun.opengl.util.j2d.TextRenderer;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

import inf119.interfaces.HudArtistIntf;
import inf119.interfaces.TextureIntf;

public class MyxHudArtist extends AbstractMyxSimpleBrick implements HudArtistIntf {

	public static final IMyxName msg = MyxUtils.createName("HudArtist");
	public static final IMyxName txtmsg = MyxUtils.createName("Texture");
	
	GL gl;
	TextureIntf tm;
	TextRenderer	texRen;
	
	private double[] landerRot = { 0,0,0 };

	public void setGL(GL g)
	{
		gl = g;
	}
	
	public void setTR(TextRenderer tr){
		texRen = tr;
	}
	
	public String formatDouble(double data, int precisionWhole, int precisionFraction)
	/*
	 * data is the value to be displayed
	 * precisionWhole and precisionFraction are positive integers
	 * truncate data to <precisionFraction> places
	 * pad data to <precisionWhole> figures
	 * 
	 * Return the value as a string that can be drawn by texRen
	 * The goal is to format all doubles to fit the same width & align their decimal places
	 * This prevents numbers from jittering in the HUD
	 */
	{
		//this will zero pad the string and leave a space for positive numbers 
		//(so it doesn't shift when there's no negative sign)
		String formatted = String.format("%1$0 "+precisionWhole+"."+precisionFraction+"f", data);
		return formatted;
	}
	
	/*
	 * Draws HUD with simulated drop shadow
	 * 
	 */
	public void drawHUD(int width, int height, double alt, double vel, double fuel, double pitch, double yaw, double roll ) 
	{
		//mod the values to the range we want
		landerRot[0] = ((pitch) % 360);
		landerRot[1] = ((roll) % 360);
		landerRot[2] = ((yaw) % 360);
		
		drawHUDText(11, 21, width, height, alt, vel, fuel, landerRot[0], landerRot[1], landerRot[2]);
		
		//gl.glEnable(GL.GL_CULL_FACE);
		gl.glCullFace(GL.GL_BACK); //we have to re-enable this because the TextRenderer class seems to disable it
	}
	
	/*
	 * Draw the HUD text
	 */
	private void drawHUDText(int xOffset, int yOffset, int width, int height, double alt, double vel, double fuel, double pitch, double yaw, double roll ) 
	{			
		//TODO:  	add units of measure to display?
		String altStr = formatDouble(alt, 8, 2);
		String velStr = formatDouble(vel, 8, 2);
		String fuelStr = formatDouble(fuel, 7, 2);
		String pitStr = formatDouble(pitch, 7, 2);
		String yawStr = formatDouble(yaw, 7, 2);
		String rollStr = formatDouble(roll, 7, 2);
		
		texRen.setColor(0.0f, 0.0f, 0.0f, 1.0f);
				
		//draws everything twice for simulated drop shadow
		//(once in green, and once in black behind
		for(int i = 0; i < 2; i++)
		{
			texRen.beginRendering(width, height);
		
			texRen.draw("Altitude:", xOffset, height-yOffset);
			texRen.draw("Velocity:", xOffset, height-yOffset-15);
			texRen.draw("Fuel:", xOffset, height-yOffset-30);
			texRen.draw("Pitch:", xOffset, height-yOffset-60);
			texRen.draw("Yaw:", xOffset, height-yOffset-75);
			texRen.draw("Roll:", xOffset, height-yOffset-90);
		
			texRen.draw(altStr, xOffset+ 60, height-yOffset);
			texRen.draw(velStr, xOffset+ 60, height-yOffset-15);
			texRen.draw(fuelStr, xOffset+ 68, height-yOffset-30);
			texRen.draw(pitStr, xOffset+ 45, height-yOffset-60);
			texRen.draw(yawStr, xOffset+ 45, height-yOffset-75);
			texRen.draw(rollStr, xOffset+ 45, height-yOffset-90);
		
			texRen.endRendering();
			
			texRen.setColor(0.0f, 1.0f, 0.0f, 1.0f);
			xOffset--;
			yOffset--;
		}
		
	}
	
	public void begin(){
		tm = (TextureIntf)MyxUtils.getFirstRequiredServiceObject(this, txtmsg);
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}

}
