package inf119.graphics;

import javax.media.opengl.GL;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

import inf119.LanderModel;
import inf119.interfaces.LanderArtistIntf;
import inf119.interfaces.TextureIntf;

public class MyxLanderArtist extends AbstractMyxSimpleBrick implements LanderArtistIntf {
	
	public static final IMyxName msg = MyxUtils.createName("LanderArtist");
	public static final IMyxName txtmsg = MyxUtils.createName("Texture");

	final static double DEFAULT_RES = 100;
	final static int DEFAULT_LOGO_TEX = 4; 
	final static double[] DEFAULT_THRUST_LOC = { 0, 0, 0 };
	final static double DEFAULT_THRUST_RAD = 1.0;
	final static double DEFAULT_FEET = 4.1;
	final static double[] DEFAULT_CENT_MASS = { 0, 0.6, 0 };
	                    

	private double LANDER_MODEL[][][] = 
	{
		//SPHERE portion:
			{  // back face
				{-0.5, 0.5, -1.5},
				{0.5, 0.5, -1.5},
				{0.5, -0.5, -1.5},
				{-0.5, -0.5, -1.5}},

			  {  // front face 
				{0.5, 0.5, 1.5},
				{-0.5, 0.5, 1.5},
				{-0.5, -0.5, 1.5},
				{0.5, -0.5, 1.5}},
				
			{	//left face
				{-1.5, 0.5, 0.5},
				{-1.5, 0.5, -0.5},
				{-1.5, -0.5, -0.5},
			    {-1.5, -0.5, 0.5}},

			  {  // right face
			    {1.5, 0.5, -0.5},
			    {1.5, 0.5, 0.5},
			    {1.5, -0.5, 0.5},
			    {1.5, -0.5, -0.5}},

			  {  // bottom face
			    {-0.5, -1.5, -0.5},
			    {0.5, -1.5, -0.5},
			    {0.5, -1.5, 0.5},
			    {-0.5, -1.5, 0.5}},
			    
			  {  // top face
				{0.5, 1.5, 0.5},
				{0.5, 1.5, -0.5},
				{-0.5, 1.5, -0.5},
				{-0.5, 1.5, 0.5}},
			  
			  {  // upper back face
				{-0.5, 1.5, -0.5},
				{0.5, 1.5, -0.5},
				{0.5, 0.5, -1.5},
				{-0.5, 0.5, -1.5}},
			
			{  // upper left face
			    {-1.5, 0.5, 0.5},
			    {-0.5, 1.5, 0.5},
			    {-0.5, 1.5, -0.5},
			    {-1.5, 0.5, -0.5}},
			    
			  {  // upper right face
				{1.5, 0.5, 0.5},
				{1.5, 0.5, -0.5},
				{0.5, 1.5, -0.5},
				{0.5, 1.5, 0.5}},


			  {  // upper front face
				{0.5, 1.5, 0.5},
				{-0.5, 1.5, 0.5},
				{-0.5, 0.5, 1.5},
				{0.5, 0.5, 1.5}},
				
			  {  // lower front face
				{0.5, -1.5, 0.5},
				{0.5, -0.5, 1.5},
				{-0.5, -0.5, 1.5},
				{-0.5, -1.5, 0.5}},
			
			  {  // lower back face
				{0.5, -1.5, -0.5},
				{-0.5, -1.5, -0.5},
				{-0.5, -0.5, -1.5},
				{0.5, -0.5, -1.5}},
			
			  {  // lower left face
				{-0.5, -1.5, -0.5},
				{-0.5, -1.5, 0.5},
				{-1.5, -0.5, 0.5},						
				{-1.5, -0.5, -0.5}},
			
			  {  // lower right face
				{0.5, -1.5, -0.5},
				{1.5, -0.5, -0.5},
				{1.5, -0.5, 0.5},
				{0.5, -1.5, 0.5}},	
				
			  {  // upper back right corner
				{0.5, 1.5, -0.5},
				{1.5, 0.5, -0.5},
				{0.5, 0.5, -1.5},
				{0.5, 1.5, -0.5}},
			
			  { // upper back left corner
				{-0.5, 1.5, -0.5},
				{-0.5, 0.5, -1.5},
				{-1.5, 0.5, -0.5},			
				{-0.5, 1.5, -0.5}},
			  
			  { // upper front left corner
				{-0.5, 1.5, 0.5},
				{-1.5, 0.5, 0.5},
				{-0.5, 0.5, 1.5},						
				{-0.5, 1.5, 0.5}},
			
			  { // upper front right corner
				{0.5, 1.5, 0.5},			
				{0.5, 0.5, 1.5},
				{1.5, 0.5, 0.5},									
				{0.5, 1.5, 0.5}},
				
			  {  // lower back right corner
				{0.5, -1.5, -0.5},
				{0.5, -0.5, -1.5},
				{1.5, -0.5, -0.5},			
				{0.5, -1.5, -0.5}},
				
			  { // lower back left corner
				{-0.5, -1.5, -0.5},						
				{-1.5, -0.5, -0.5},
				{-0.5, -0.5, -1.5},
				{-0.5, -1.5, -0.5}},
				  
			  { // lower front left corner
				{-0.5, -1.5, 0.5},
				{-0.5, -0.5, 1.5},
				{-1.5, -0.5, 0.5},									
				{-0.5, -1.5, 0.5}},
				
		      { // lower front right corner
				{0.5, -1.5, 0.5},
				{1.5, -0.5, 0.5},									
				{0.5, -0.5, 1.5},
				{0.5, -1.5, 0.5}},
				
			  {  // back left edge
				{-1.5, 0.5, -0.5},			
				{-0.5, 0.5, -1.5},
				{-0.5, -0.5, -1.5},
				{-1.5, -0.5, -0.5}},
			
			  {  // back right edge
				{1.5, 0.5, -0.5},			
				{1.5, -0.5, -0.5},
				{0.5, -0.5, -1.5},
				{0.5, 0.5, -1.5}},
				
			  {  // front right edge
				{1.5, 0.5, 0.5},
				{0.5, 0.5, 1.5},
				{0.5, -0.5, 1.5},
				{1.5, -0.5, 0.5}},
			
			  {  // front left edge
				{-1.5, 0.5, 0.5},
				{-1.5, -0.5, 0.5},
				{-0.5, -0.5, 1.5},
				{-0.5, 0.5, 1.5}},	
				
		//BOX portion:
			{  // back face
				{-1.0, -1.5, -1.0},
				{1.0, -1.5, -1.0},
				{1.0, -3.1, -1.0},
				{-1.0, -3.1, -1.0}},

			  {  // front face 
				{1.0, -1.5, 1.0},
				{-1.0, -1.5, 1.0},
				{-1.0, -3.1, 1.0},
				{1.0, -3.1, 1.0}},
				
			{	//left face
				{-1.0, -1.5, 1.0},
				{-1.0, -1.5, -1.0},
				{-1.0, -3.1, -1.0},
			    {-1.0, -3.1, 1.0}},

			  {  // right face
			    {1.0, -1.5, -1.0},
			    {1.0, -1.5, 1.0},
			    {1.0, -3.1, 1.0},
			    {1.0, -3.1, -1.0}},

			  {  // bottom face
			    {-1.0, -3.1, -1.0},
			    {1.0, -3.1, -1.0},
			    {1.0, -3.1, 1.0},
			    {-1.0, -3.1, 1.0}},
			    
		    {  // top face
				{1.0, -1.5, 1.0},
				{1.0, -1.5, -1.0},
				{-1.0, -1.5, -1.0},
				{-1.0, -1.5, 1.0}},
			  
		//LEG 1:
			{ //left side
				{1.0, -2.8, 1.0},
				{0.7, -3.1, 1.0},
				{1.9, -4.1, 2.0},
				{2.1, -4.1, 2.0}},
				
			{ //right side
				{1.0, -3.1, 0.7},
				{1.0, -2.8, 1.0},
				{1.9, -4.1, 2.0},
				{2.1, -4.1, 2.0}},	
				
			{ //bottom side
				{0.7, -3.1, 1.0},
				{1.0, -3.1, 0.7},			
				{2.1, -4.1, 2.0},
				{1.9, -4.1, 2.0}},	
				
			{ //foot top
				{1.7, -4.1, 1.7},
				{1.7, -4.1, 2.3},			
				{2.3, -4.1, 2.3},
				{2.3, -4.1, 1.7}},
				
			{ //foot bottom
				{1.7, -4.1, 1.7},
				{2.3, -4.1, 1.7},
				{2.3, -4.1, 2.3},
				{1.7, -4.1, 2.3}},
		
		//	LEG 2:
			{ //left side
				{-1.0, -2.8, 1.0},
				{-2.1, -4.1, 2.0},
				{-1.9, -4.1, 2.0},
				{-0.7, -3.1, 1.0}},
					
			{ //right side
				{-1.0, -3.1, 0.7},
				{-2.1, -4.1, 2.0},
				{-1.9, -4.1, 2.0},
				{-1.0, -2.8, 1.0}},	
					
			{ //bottom side
				{-0.7, -3.1, 1.0},
				{-1.9, -4.1, 2.0},
				{-2.1, -4.1, 2.0},
				{-1.0, -3.1, 0.7}},	
					
			{ //foot top
				{-1.7, -4.1, 1.7},
				{-2.3, -4.1, 1.7},
				{-2.3, -4.1, 2.3},
				{-1.7, -4.1, 2.3}},
					
			{ //foot bottom
				{-1.7, -4.1, 1.7},
				{-1.7, -4.1, 2.3},
				{-2.3, -4.1, 2.3},
				{-2.3, -4.1, 1.7}},
			    
		// LEG 3:
			{ //left side
				{-1.0, -2.8, -1.0},
				{-0.7, -3.1, -1.0},
				{-1.9, -4.1, -2.0},
				{-2.1, -4.1, -2.0}},
						
			{ //right side
				{-1.0, -3.1, -0.7},
				{-1.0, -2.8, -1.0},
				{-1.9, -4.1, -2.0},
				{-2.1, -4.1, -2.0}},	
						
			{ //bottom side
				{-0.7, -3.1, -1.0},
				{-1.0, -3.1, -0.7},
				{-2.1, -4.1, -2.0},
				{-1.9, -4.1, -2.0}},	
						
			{ //foot top
				{-1.7, -4.1, -1.7},
				{-1.7, -4.1, -2.3},
				{-2.3, -4.1, -2.3},
				{-2.3, -4.1, -1.7}},
						
			{ //foot bottom
				{-1.7, -4.1, -1.7},
				{-2.3, -4.1, -1.7},
				{-2.3, -4.1, -2.3},
				{-1.7, -4.1, -2.3}},
				
		// LEG 4:
			{ //left side
				{1.0, -2.8, -1.0},
				{2.1, -4.1, -2.0},
				{1.9, -4.1, -2.0},
				{0.7, -3.1, -1.0}},
							
			{ //right side
				{1.0, -3.1, -0.7},
				{2.1, -4.1, -2.0},
				{1.9, -4.1, -2.0},
				{1.0, -2.8, -1.0}},	
							
			{ //bottom side
				{0.7, -3.1, -1.0},
				{1.9, -4.1, -2.0},
				{2.1, -4.1, -2.0},
				{1.0, -3.1, -0.7}},	
							
			{ //foot top
				{1.7, -4.1, -1.7},
				{2.3, -4.1, -1.7},
				{2.3, -4.1, -2.3},
				{1.7, -4.1, -2.3}},
							
			{ //foot bottom
				{1.7, -4.1, -1.7},
				{1.7, -4.1, -2.3},
				{2.3, -4.1, -2.3},
				{2.3, -4.1, -1.7}},
	
	};	
	
	GL gl;
	TextureIntf tm;
	LanderModel lm;

	public MyxLanderArtist()
	{
		//tm = t;
		int[] tex = { 2, 3 };
		lm = new LanderModel(DEFAULT_RES, tex, LANDER_MODEL, DEFAULT_LOGO_TEX, DEFAULT_THRUST_LOC, DEFAULT_THRUST_RAD, DEFAULT_FEET, DEFAULT_CENT_MASS);
	}
	
	public void setGL(GL g)
	{
		gl = g;
	}
	
	/*
	 *  allows for different model to be used
	 */
	public void setLanderModel(LanderModel lm)
	{
		this.lm = lm;
	}
	
	/*
	 * Utility function to draw faces of the model
	 */
	private void drawLanderFace(GL gl, int face, double[][][] model)
	{	
		gl.glBegin(GL.GL_POLYGON);
		gl.glTexCoord2d( 1.0, 1.0 );
		gl.glVertex3d(model[face][0][0], model[face][0][1], model[face][0][2]);
	    gl.glTexCoord2d( 0.0, 1.0 );
	    gl.glVertex3d(model[face][1][0], model[face][1][1], model[face][1][2]);
	    gl.glTexCoord2d( 0.0, 0.0 );
	    gl.glVertex3d(model[face][2][0], model[face][2][1], model[face][2][2]);
	    gl.glTexCoord2d( 1.0, 0.0 );
	    gl.glVertex3d(model[face][3][0], model[face][3][1], model[face][3][2]);
	    gl.glEnd();	
	}
	
	/*
	 * This method draws the lander model
	 * 
	 */
	public void drawLander(double[] pos, double[] rotation)
	{	
		gl.glEnable(GL.GL_CULL_FACE);
		gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_FILL);
		int i = 0;
		final int LANDER_1 = lm.texFaces, LANDER_2 = lm.getVertices().length;
	    	
		gl.glEnable( GL.GL_TEXTURE_2D );
    	gl.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL);

		gl.glPushMatrix();
		
		//calculate position of lander in units, adjusted for center of mass
		double[] unitPos = new double[3];
		for (i = 0; i < unitPos.length; i++){
			unitPos[i] = MyxGraphicsMaster.metersToUnits(pos[i]-lm.centMass[i]);			
		}
		gl.glTranslated(unitPos[0], unitPos[1], unitPos[2]);
		
		//lander orientation
		gl.glRotated(-rotation[0], 1.0, 0.0, 0.0);
		gl.glRotated(-rotation[1], 0.0, 1.0, 0.0);
		gl.glRotated(-rotation[2], 0.0, 0.0, 1.0);

		//lander scale
		double newScale = MyxGraphicsMaster.metersToUnits(lm.resolution);
		gl.glScaled(newScale, newScale, newScale);
		
		/*
		 * NOTE:  the code below should change to loop through all different textures and faces eventually
		 * right now it just assumes 2 textures 
		*/
		//Draw lander faces with first texture
		gl.glBindTexture( GL.GL_TEXTURE_2D, tm.getTexID(lm.textureIDs[0]) );
		for (i = 0; i < LANDER_1; i++){
			drawLanderFace(gl, i, lm.getVertices());
		}
		//Draw lander faces with second texture
		gl.glBindTexture( GL.GL_TEXTURE_2D, tm.getTexID(lm.textureIDs[1]) );
		for (; i < LANDER_2; i++){
			drawLanderFace(gl, i, lm.getVertices());
		}
		
		gl.glPopMatrix();
	}	
	
	public double[] getThrustLoc(){
		return lm.thrustLoc;
	}
	
	public double getResolution(){
		return lm.resolution;
	}
	
	public double[] getCentMass(){
		return lm.centMass;
	}

	public void begin(){
		tm = (TextureIntf)MyxUtils.getFirstRequiredServiceObject(this, txtmsg);
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}
}
