package inf119.graphics;

import javax.media.opengl.GL;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

import inf119.interfaces.SkyboxArtistIntf;
import inf119.interfaces.TextureIntf;

public class MyxSkyboxArtist extends AbstractMyxSimpleBrick implements SkyboxArtistIntf {

	public static final IMyxName msg = MyxUtils.createName("SkyboxArtist");
	public static final IMyxName txtmsg = MyxUtils.createName("Texture");
	
	final double NUDGE = 0.01;
	final static double CUBE_V[][][] =
	{
	  {
	    {-1.0, -1.0, -1.0},
	    {-1.0, 1.0, -1.0},
	    {-1.0, 1.0, 1.0},
	    {-1.0, -1.0, 1.0}},

	  {
	    {1.0, 1.0, 1.0},
	    {1.0, 1.0, -1.0},
	    {1.0, -1.0, -1.0},
	    {1.0, -1.0, 1.0}},

	  {
	    {-1.0, -1.0, -1.0},
	    {-1.0, -1.0, 1.0},
	    {1.0, -1.0, 1.0},
	    {1.0, -1.0, -1.0}},

	  {
	    {1.0, 1.0, 1.0},
	    {-1.0, 1.0, 1.0},
	    {-1.0, 1.0, -1.0},
	    {1.0, 1.0, -1.0}},

	  {
	    {-1.0, -1.0, -1.0},
	    {1.0, -1.0, -1.0},
	    {1.0, 1.0, -1.0},
	    {-1.0, 1.0, -1.0}},

	  {
	    {1.0, 1.0, 1.0},
	    {1.0, -1.0, 1.0},
	    {-1.0, -1.0, 1.0},
	    {-1.0, 1.0, 1.0}}
	};
	
	GL gl;
	TextureIntf tm;
	private double skyboxLength; //The length of the box in GL Units

	public MyxSkyboxArtist()
	{
		skyboxLength = 15;
	}
	
	public void setGL(GL g)
	{
		gl = g;
	}
	
	public void setSkyboxLength(double length)
	/*
	 * Takes the length in "meters" and stores it in GL coordinates
	 */
	{
		skyboxLength = MyxGraphicsMaster.metersToUnits(length);
		System.out.println("Skybox Length: " + skyboxLength);
	}
	
	public void drawSkybox()
	{
	    // set polygon fill mode
	    gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_FILL);
	    //gl.glEnable(GL.GL_COLOR_MATERIAL);
	    //gl.glColorMaterial(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT_AND_DIFFUSE);

	    // draw each cube face as an individual polygon
    	gl.glEnable( GL.GL_TEXTURE_2D );
    	gl.glBindTexture( GL.GL_TEXTURE_2D, tm.getTexID(0) );
    	gl.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL);

		  gl.glPushMatrix();
		  gl.glScaled(skyboxLength, skyboxLength, skyboxLength);

	    for (int i = 0; i < CUBE_V.length; i++) {
		  gl.glBegin(GL.GL_POLYGON);
    	  gl.glTexCoord2d( 0, 0 );
    	  gl.glVertex3d(CUBE_V[i][0][0], CUBE_V[i][0][1], CUBE_V[i][0][2]);
    	  gl.glTexCoord2d( 0, skyboxLength );
    	  gl.glVertex3d(CUBE_V[i][1][0], CUBE_V[i][1][1], CUBE_V[i][1][2]);
    	  gl.glTexCoord2d( skyboxLength, skyboxLength );
    	  gl.glVertex3d(CUBE_V[i][2][0], CUBE_V[i][2][1], CUBE_V[i][2][2]);
    	  gl.glTexCoord2d( skyboxLength, 0 );
    	  gl.glVertex3d(CUBE_V[i][3][0], CUBE_V[i][3][1], CUBE_V[i][3][2]);
	      gl.glEnd();
	    }
	      gl.glPopMatrix();
	}
	
	/*
	 * draws the Earth on the skybox
	 */
	public void drawEarth()
	{
		gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_FILL);
	    gl.glEnable( GL.GL_TEXTURE_2D );
	    gl.glBindTexture( GL.GL_TEXTURE_2D, tm.getTexID(4) );
	    gl.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_MODULATE);
	    
	    gl.glPushMatrix();
	    //scale it down so it doesn't take up the whole skybox 
	    gl.glScaled(skyboxLength, skyboxLength/15, skyboxLength/15);
	      
	    
	    gl.glBegin(GL.GL_POLYGON);
	      gl.glColor3d(0.7, 0.7, 0.7);
	      
	      //draw the earth image (NUDGE adjusts the location so the earth does not actually intersect with the 
	      //skybox wall
	      gl.glTexCoord2d( 0, 0 );
	      gl.glVertex3d(CUBE_V[0][0][0] + NUDGE, CUBE_V[0][0][1], CUBE_V[0][0][2]);
    	  gl.glTexCoord2d( 0, 1 );
    	  gl.glVertex3d(CUBE_V[0][1][0] + NUDGE, CUBE_V[0][1][1], CUBE_V[0][1][2]);
    	  gl.glTexCoord2d( 1, 1 );
    	  gl.glVertex3d(CUBE_V[0][2][0] + NUDGE, CUBE_V[0][2][1], CUBE_V[0][2][2]);
    	  gl.glTexCoord2d( 1, 0 );
    	  gl.glVertex3d(CUBE_V[0][3][0] + NUDGE, CUBE_V[0][3][1], CUBE_V[0][3][2]);
	    gl.glEnd();
	      
	    gl.glPopMatrix();
	}

	public void begin(){
		tm = (TextureIntf)MyxUtils.getFirstRequiredServiceObject(this, txtmsg);
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}
}
